#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform sampler2D albedoTexture;
uniform sampler2D metallicTexture;
uniform sampler2D roughnessTexture;
uniform sampler2D aoTexture;

uniform float brightness;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL[3];

 const float PI = 3.14159265359;

vec3 fresnelSchlick(float cosTheta, vec3 F0)
{
    return F0 + (1.0 - F0) * pow(clamp(1.0 - cosTheta, 0.0, 1.0), 5.0);
}  

float DistributionGGX(vec3 N, vec3 H, float roughness)
{
    float a      = roughness*roughness;
    float a2     = a*a;
    float NdotH  = max(dot(N, H), 0.0);
    float NdotH2 = NdotH*NdotH;
	
    float num   = a2;
    float denom = (NdotH2 * (a2 - 1.0) + 1.0);
    denom = PI * denom * denom;
	
    return num / denom;
}

float GeometrySchlickGGX(float NdotV, float roughness)
{
    float r = (roughness + 1.0);
    float k = (r*r) / 8.0;

    float num   = NdotV;
    float denom = NdotV * (1.0 - k) + k;
	
    return num / denom;
}

float GeometrySmith(vec3 N, vec3 V, vec3 L, float roughness)
{
    float NdotV = max(dot(N, V), 0.0);
    float NdotL = max(dot(N, L), 0.0);
    float ggx2  = GeometrySchlickGGX(NdotV, roughness);
    float ggx1  = GeometrySchlickGGX(NdotL, roughness);
	
    return ggx1 * ggx2;
}

void main() {	
	vec3 N = normalize(vert.normal_world);
	vec3 V = normalize(camera_world - vert.position_world);

	vec2 uv = vec2(vert.uv.x, 1-vert.uv.y); 
	vec3 albedo = texture(albedoTexture, uv).rgb;
	float metallic  = texture(metallicTexture, uv).r;
    float roughness = texture(roughnessTexture, uv).r;
    float ao        = texture(aoTexture, uv).r;
	
	vec3 F0 = vec3(0.04); 
	F0      = mix(F0, albedo, metallic);

	vec3 Lo = vec3(0.0);
	for(int i = 0; i < pointL.length; i++) {
		vec3 L = normalize(pointL[i].position - vert.position_world);
		//vec3 L = normalize(pointL.position - vert.position_world);
		vec3 H = normalize(V + L);
  
		float distance    = length(pointL[i].position -  vert.position_world);
		//float distance    = length(pointL.position -  vert.position_world);
		float attenuation = 1.0 / (distance * distance);
		vec3 radiance     = pointL[i].color * attenuation;
		//vec3 radiance     = pointL.color * attenuation; 

		vec3 F  = fresnelSchlick(max(dot(H, V), 0.0), F0);
		float NDF = DistributionGGX(N, H, roughness);        
        float G   = GeometrySmith(N, V, L, roughness); 

		vec3 numerator    = NDF * G * F;
		float denominator = 4.0 * max(dot(N, V), 0.0) * max(dot(N, L), 0.0)  + 0.0001;
		vec3 specular     = numerator / denominator;  

		vec3 kS = F;
		vec3 kD = vec3(1.0) - kS;
  
		kD *= 1.0 - metallic;	
 
		float NdotL = max(dot(N, L), 0.0);                
        Lo += (kD * albedo / PI + specular) * radiance * NdotL; 
	}

	//directional light

	vec3 L = normalize(-dirL.direction);
	vec3 H = normalize(V + L);
  
	vec3 radiance     = dirL.color;

	vec3 F  = fresnelSchlick(max(dot(H, V), 0.0), F0);
	float NDF = DistributionGGX(N, H, roughness);        
    float G   = GeometrySmith(N, V, L, roughness); 

	vec3 numerator    = NDF * G * F;
	float denominator = 4.0 * max(dot(N, V), 0.0) * max(dot(N, L), 0.0)  + 0.0001;
	vec3 specular     = numerator / denominator;  

	vec3 kS = F;
	vec3 kD = vec3(1.0) - kS;
  
	kD *= 1.0 - metallic;	
 
	float NdotL = max(dot(N, L), 0.0);                
    Lo += (kD * albedo / PI + specular) * radiance * NdotL; 

	vec3 ambient = vec3(0.03) * albedo * ao;
    vec3 colorB = ambient + Lo;
	
    colorB = colorB / (colorB + vec3(1.0));
    colorB = pow(colorB, vec3(1.0/2.2)); 
	
	vec3 testColor = vec3(1,0,1);
   
    color = vec4(colorB, 1.0) * brightness; 
}


